import logging
from logging.handlers import RotatingFileHandler
import os
import sys


class DuplicateFilter(logging.Filter):
    """ a filter to suppress repeating log messages """
    def __init__(self):
        logging.Filter.__init__(self)
        self.last_log = None

    def filter(self, record):
        current_log = record.msg
        if current_log != self.last_log:
            self.last_log = current_log
            return True
        return False


class MyLogger(logging.Logger):
    """
    Preconfigured logger class with parameters:
    :param log_file_name: name of log file to write to
    :param log_level:
    :param log_file_size_limit_bytes: if the limit exceeds file will be backed up and cleared
    :return: preconfigured logger object
    """
    def __init__(self, log_file_name="logfile.log", log_level=logging.DEBUG, logger_name='my logger',
                 log_file_size_limit_bytes=1024*1024, log_file_dir="logs", use_script_dir=False, prefix=""):

        logging.Logger.__init__(self, logger_name)

        logs_formatter = logging.Formatter(f'%(asctime)s [%(levelname)s]{prefix} %(message)s')
        self.addFilter(DuplicateFilter())
        self.setLevel(log_level)

        if log_file_name:
            self.log_dir_path = self._get_log_dir(log_file_dir, use_script_dir)
            self.log_file_path = os.path.join(self.log_dir_path, log_file_name)
            if not os.path.isdir(self.log_dir_path):
                os.mkdir(self.log_dir_path)
            self.log_file = RotatingFileHandler(self.log_file_path, maxBytes=log_file_size_limit_bytes, backupCount=5)
            self.log_file.setFormatter(logs_formatter)
            self.addHandler(self.log_file)

    @staticmethod
    def _get_log_dir(log_file_dir, use_script_dir):
        if use_script_dir:
            base_dir = os.path.dirname(os.path.realpath(__file__))
            return os.path.join(base_dir, log_file_dir)
        else:
            return log_file_dir

    def write(self, message):
        """
        Write method to provide a way of redirecting stdout.
        """
        if "error" in message.lower():
            self.error(message)
        else:
            self.debug(message)

    def flush(self):
        """
        Flush method to provide a way of redirecting stdout.
        """
        self.error(sys.stderr)



import json
from threading import Thread
from time import sleep

import requests

import settings
from .my_logger import MyLogger
from .database_manager import DatabaseManager
from .road_map import RoadMap


class RsiDataParser:
    base_url = settings.BASE_URL
    ship_upgrades_url = settings.SHIP_UPGRADES_URL
    ships_matrix_url = settings.SHIPS_MATRIX_URL
    game_packages_url = settings.GAME_PACKAGES_URL
    ships_matrix_keys = settings.SHIPS_MATRIX_KEYS

    def __init__(self, auto_update_period=0, log_file='rsi_parser.log', database_manager=None):
        self.logger = MyLogger(log_file_name=log_file, logger_name="RSI parser logger", prefix="[RSI_PARSER]")
        if database_manager:
            self.database = database_manager
        else:
            self.database = DatabaseManager(log_file=log_file)

        self.ships = {}
        self.build_ships_base()
        if auto_update_period:
            self.auto_update_period = auto_update_period
            self.auto_update_thread = Thread(target=self.update_prices_periodically)
            self.auto_update_thread.start()

        self.road_map = RoadMap(log_file=log_file, database_manager=database_manager)

    def try_to_request_get(self, url):
        try:
            response = requests.get(url)
            if response.status_code == 200:
                return response
            else:
                self.logger.warning("Could not get data from RSI website. HTTP status code %s." % response.status_code)
        except requests.exceptions.ConnectionError as err:
            self.logger.warning("Could not request RSI website due to following error:\n%s" % str(err))

    def get_ships_prices_string(self):
        response = self.try_to_request_get(self.ship_upgrades_url)
        if response:
            results_html = response.text
            for line in results_html.split('\n'):
                if "RSI.ShipUpgrade.MainView" in line:
                    return line.split("fromShips: ")[1].split(", toShips: ")[0]
            else:
                self.logger.error("Didn't find 'RSI.ShipUpgrade.MainView' string on page.")

    def get_ships_prices(self):
        prices_string = self.get_ships_prices_string()
        if prices_string:
            return json.loads(prices_string)

    def get_ships_matrix(self):
        response = self.try_to_request_get(self.ships_matrix_url)
        if response:
            result = response.json()
            if result.get("success") == 1:
                return result["data"]
            else:
                self.logger.error("Ship Matrix didn't return 'success' flag.")

    def build_ships_base(self):
        ship_matrix = self.get_ships_matrix()
        if ship_matrix:
            for ship in ship_matrix:
                ship_name = ship["name"].lower()
                self.ships[ship_name] = {data_key: ship[data_key] for data_key in self.ships_matrix_keys}
                self.ships[ship_name]["manufacturer"] = ship["manufacturer"]["name"]
                self.ships[ship_name]["manufacturer_code"] = ship["manufacturer"]["code"]
            self.update_ships_prices()
            self.database.save_rsi_data(self.ships)
        else:
            self.ships = self.database.get_rsi_data()

    def update_ships_prices(self):
        prices = self.get_ships_prices()
        if prices:
            for ship in prices:
                ship_name = ship["name"].lower()
                self.ships[ship_name]["price"] = ship["msrp"]

    def update_prices_periodically(self):
        while self.auto_update_period > 0:
            self.update_ships_prices()
            sleep(self.auto_update_period)

    def get_ship(self, ship_name):
        ship = self.ships.get(ship_name.lower())
        if ship:
            self.shorten_manufacturer_name(ship)
            return ship

    def get_ships_by_query(self, query):
        query = query.lower()
        result = []
        for ship in self.ships.values():
            if query in ship["name"].lower() \
                    or query in ship["manufacturer"].lower() \
                    or query in ship["manufacturer_code"].lower():
                self.shorten_manufacturer_name(ship)
                result.append(ship)
        return result

    @staticmethod
    def shorten_manufacturer_name(ship, db_ship=None):
        if db_ship is None:
            db_ship = ship
        if len(db_ship["manufacturer"]) > 20:
            ship["manufacturer"] = db_ship["manufacturer_code"]
        else:
            ship["manufacturer"] = db_ship["manufacturer"]

    def verify_ship(self, ship):
        db_ship = self.get_ship(ship.get('name'))
        if db_ship:
            self.shorten_manufacturer_name(ship, db_ship)
            return ship

    def verify_ships(self, ships_data):
        verified_ships = []
        invalid_ships = []
        for ship in ships_data:
            ship = self.verify_ship(ship)
            if ship:
                verified_ships.append(ship)
            else:
                invalid_ships.append({'name': ship['name'], 'manufacturer': ship['manufacturer']})
        return verified_ships, invalid_ships

    def get_game_packages(self):
        return self.try_to_request_get(self.game_packages_url).text

    def get_updated_versions(self):
        self.road_map.update_road_map()
        return self.road_map.current_versions

    def _iterate_new_forum_threads(self, hits):
        for hit in hits:
            thread = hit["details"]["thread"]
            thread_url = settings.BASE_URL + "/spectrum/community/SC/forum/%s/thread/%s" % (
                thread["channel_id"], thread["slug"])
            if self.forum_query in thread["subject"]:
                if self.database.thread_is_new(thread["channel_id"], thread["subject"], thread_url):
                    yield thread_url

    @property
    def forum_query(self):
        current_release = self.road_map.current_versions['live']
        first_number, second_number = current_release.split('.')[0:2]
        second_number = str(int(second_number) + 1)
        next_release = ".".join((first_number, second_number))
        return "Star Citizen Alpha %s" % next_release

    def get_forum_release_messages(self):
        payload = settings.FORUM_SEARCH_PAYLOAD
        payload["text"] = self.forum_query
        r = requests.post(settings.FORUM_SEARCH_URL, data=json.dumps(payload))
        data = r.json().get("data")

        if data:
            hits = data.get("hits")
            if hits and hits["total"]:
                return list(self._iterate_new_forum_threads(hits["hits"]))


if __name__ == '__main__':
    print(RoadMap().current_versions)



import languages

# Loggers
LOG_FILE = "dastro_bot.log"
LOGGER_NAME = "Star Citizen Discord Astro-Bot logger"

# language
messages = languages.messages_en
additional_commands = languages.commands_en

# server
CHANNELS = {
    'main': '000000000000000000',
    'lobby': '000000000000000000',
    'recruitment': '000000000000000000',
}

# Database manager
DATABASE_NAME = "database.sqlite"
DATABASE_DIALECT = 'sqlite:///%s'

# RSI DATA PARSER
BASE_URL = "https://robertsspaceindustries.com"
SHIP_DATA_HEADERS = ['manufacturer', 'name', 'price', 'focus', 'production_status', 'length', 'beam', 'height',
                     'size', 'mass', 'cargocapacity', 'min_crew', 'max_crew', 'scm_speed', 'afterburner_speed']
SHIP_UPGRADES_URL = BASE_URL + "/pledge/ship-upgrades"
SHIPS_MATRIX_URL = BASE_URL + "/ship-matrix/index"
GAME_PACKAGES_URL = BASE_URL + "/pledge/game-packages"
SHIPS_MATRIX_KEYS = ["name", "focus", "url", "production_status", "length", "beam", "height", "size", "mass",
                     "cargocapacity", "min_crew", "max_crew", "scm_speed", "afterburner_speed"]

API_INIT_URL = BASE_URL + "/roadmap/board/1-Star-Citizen"
ROAD_MAP_URL = BASE_URL + "/api/roadmap/v1/boards/1"
FORUM_SEARCH_URL = BASE_URL + "/api/spectrum/search/content/extended"
FORUM_SEARCH_PAYLOAD = {
    "community_id":"1",
    "highlight_role_id":"2",
    "type": ["","op"],
    "text": "",
    "page": 1,
    "sort": "latest",
    "range": "week",
    "visibility": "nonerased"
}

# Price reporter
REPORT_SHIP_PRICE_LIST = [
            ("Cutlass Black", 100),
            ("Avenger Titan", 50),
        ]

# Trade Assistant
TRADE_GOOGLE_SCOPES = 'https://www.googleapis.com/auth/spreadsheets.readonly'
TRADE_SPREADSHEET_ID = '16vZzeCHnDIVRmKeqN4VHqy5Rm3nkkLrsig2odQTxuNo'
TRADE_SPREADSHEET_CELLS_RANGE = 'B3:AF31'
TRADE_SC_RELEASE = '3.2'



import requests

import settings
from .my_logger import MyLogger
from .database_manager import DatabaseManager


class RoadMap:
    api_init_url = settings.API_INIT_URL
    road_map_url = settings.ROAD_MAP_URL
    forum_search_url = settings.FORUM_SEARCH_URL

    def __init__(self, log_file='road_map.log', database_manager=None):
        self.logger = MyLogger(log_file_name=log_file, logger_name="Road Map logger", prefix="[ROAD_MAP]")
        if database_manager:
            self.database = database_manager
        else:
            self.database = DatabaseManager(log_file=log_file)

        self.releases = []
        self.categories = {}
        self.current_versions = {}
        self.update_road_map()

    @staticmethod
    def split_long_string_to_lines(input_text, base_length=20):
        if len(input_text) > 2*base_length:
            split_index = input_text.find(" ", base_length)
            input_text = input_text[:split_index] + "\n  " + input_text[split_index:]
        if len(input_text) > 3*base_length:
            split_index = input_text.find(" ", 2*base_length)
            input_text = input_text[:split_index] + "\n  " + input_text[split_index:]
        return input_text

    def get_road_map_response(self):
        try:
            with requests.Session() as session:
                session.get(self.api_init_url)
                session.headers.update({'x-rsi-token': session.cookies.get('Rsi-Token')})
                response = session.get(self.road_map_url)
            if response.status_code == 200:
                response_json = response.json()
                if response_json.get('success') == 1:
                    return response_json
                else:
                    self.logger.error("Road Map didn't return 'success' flag.")
            else:
                self.logger.warning("Could not get data from Road Map. HTTP status code %s." % response.status_code)
        except requests.exceptions.ConnectionError as err:
            self.logger.warning("Could not download Road-map from RSI website due to following error:\n%s" % str(err))

    def update_road_map(self):
        response = self.get_road_map_response()
        if response:
            self.releases = response['data'].get('releases')
            self.categories = self._get_categories_structure(response['data'].get('categories'))
            self.current_versions = self._get_current_versions(response['data'].get('description'))
            self.database.save_road_map(self.releases, self.categories, self.current_versions)
        else:
            self.releases, self.categories, self.current_versions = self.database.get_road_map()

    @staticmethod
    def _get_current_versions(version_message):
                live_key = "Live Version: "
                live = version_message.find(live_key) + len(live_key)
                ptu_key = "PTU Version: "
                ptu = version_message.find(ptu_key) + len(ptu_key)
                return {
                            'live': version_message[live:live+5],
                            'ptu': version_message[ptu:]
                       }

    @staticmethod
    def _get_categories_structure(rsi_categories):
        return {
            category.get('name').split()[0].lower(): category for category in rsi_categories
        }

    def _get_category_name(self, category_id):
        for category in self.categories.values():
            if int(category['id']) == int(category_id):
                return category['name']

    def get_releases(self):
        return [
            {
                'Release': release.get('name'),
                'Status': release.get('description')
            }
            for release in self.releases]

    def get_release_details(self, name):
        release = None
        for item in self.releases:
            if item.get('name') == name:
                release = item
                break
        if release is not None:
            result = {}
            for card in release.get('cards'):
                category_name = self._get_category_name(card['category_id'])
                value = [self.split_long_string_to_lines(card['name']),
                         self.split_long_string_to_lines(card['description'], 40)]
                result.setdefault(category_name, [value]).append(value)
            return result

    def get_category_details(self, slug):
        result = {}
        category = self.categories.get(slug)
        if category is not None:
            for release in self.releases:
                for card in release.get('cards'):
                    if int(category['id']) == int(card['category_id']):
                        release_header = release.get('name') + " " + release.get('description')
                        value = [self.split_long_string_to_lines(card['name']),
                                 self.split_long_string_to_lines(card['description'], 40)]
                        result.setdefault(release_header, [value]).append(value)
            return result

    def get_release_category_details(self, release_name, category_slug):
        release_details = self.get_release_details(release_name)
        if release_details is not None:
            category = self.categories.get(category_slug)
            if category is not None:
                items = release_details.get(category.get('name'))
                return {"%s %s" % (release_name, category.get('name')): items}

    def get(self):
        result = {}
        for release in self.releases:
            for card in release.get('cards'):
                category_name = self._get_category_name(card['category_id'])
                release_header = release.get('name') + " " + release.get('description')
                value = [self.split_long_string_to_lines(card['name']),
                         self.split_long_string_to_lines(card['description'], 40)]
                result.setdefault(" | ".join((release_header, category_name)), [value]).append(value)
        return result

    def get_releases_and_categories(self):
        return [
                ["VERSIONS"] + [release.get('name') for release in self.releases],
                ["CATEGORIES"] + list(self.categories.keys())
        ]



from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy import Column, Integer, String, Text, Boolean, ForeignKey
from sqlalchemy.orm import relationship


Base = declarative_base()


class Member(Base):
    __tablename__ = 'members'

    id = Column(Integer, primary_key=True)
    name = Column(String(77))
    discord_id = Column(String(77), unique=True)
    ships = relationship("Ship", cascade="all, delete-orphan", back_populates="owner")

    def __repr__(self):
        return "Member %s" % self.name


class Ship(Base):
    __tablename__ = 'ships'

    id = Column(Integer, primary_key=True)
    manufacturer = Column(String(77))
    name = Column(String(77))
    lti = Column(Boolean)
    package_id = Column(String(24))
    owner_id = Column(Integer, ForeignKey('members.id'))
    owner = relationship("Member", back_populates="ships")

    def __repr__(self):
        return "%s owned by %s" % (self.name, self.owner.discord_id)

    def get_basic_dict(self):
        return {k: self.__dict__.get(k, None) for k in ('name', 'manufacturer', 'lti')}

    def get_full_dict(self):
        result = self.get_basic_dict()
        result['owner'] = self.owner.name
        return result


class Version(Base):
    __tablename__ = 'versions'

    id = Column(Integer, primary_key=True)
    name = Column(String(77), unique=True)
    value = Column(String(77))


class RsiData(Base):
    __tablename__ = 'rsi_data'

    id = Column(Integer, primary_key=True)
    ships = Column(Text)


class RoadMap(Base):
    __tablename__ = 'road_map'

    id = Column(Integer, primary_key=True)
    releases = Column(Text)
    categories = Column(Text)
    current_versions = Column(Text)


class TradeData(Base):
    __tablename__ = 'trade_data'

    id = Column(Integer, primary_key=True)
    locations = Column(Text)
    prices = Column(Text)


class FoundForumThreads(Base):
    __tablename__ = 'forum_threads'

    id = Column(Integer, primary_key=True)
    subject = Column(Text)
    url = Column(Text)


class RsiLatestVideo(Base):
    __tablename__ = 'rsi_latest_video'

    id = Column(Integer, primary_key=True)
    url = Column(Text, unique=True)



import json

from googleapiclient.discovery import build
from httplib2 import Http
from oauth2client import file, client, tools

import settings
from .my_logger import MyLogger
from .database_manager import DatabaseManager


class PricesStructure:
    spreadsheet_id = settings.TRADE_SPREADSHEET_ID

    def __init__(
            self,
            release=settings.TRADE_SC_RELEASE,
            cells_range=settings.TRADE_SPREADSHEET_CELLS_RANGE,
            log_file="trade_assistant.log",
            database_manager=None
    ):
        self.logger = MyLogger(log_file_name=log_file, logger_name="Trade Assistant logger", prefix="[TRADE]")
        if database_manager:
            self.database = database_manager
        else:
            self.database = DatabaseManager(log_file=log_file)
        self.release = release
        self.range_name = '%s!%s' % (release, cells_range)
        self.locations = None
        self.prices = {}
        self._initiate_database()

    @staticmethod
    def _fill_missing_blank_cells(values):
        full_length = None
        for row in values:
            if 'Buy' in row:
                full_length = len(row)
                break
        for row in values:
            too_short_by = full_length - len(row)
            row += [''] * too_short_by
        return values

    def _get_spreadsheet_rows(self):
        store = file.Storage('google_token.json')
        credentials = store.get()
        if not credentials or credentials.invalid:
            flow = client.flow_from_clientsecrets('google_credentials.json', settings.TRADE_GOOGLE_SCOPES)
            credentials = tools.run_flow(flow, store)
        service = build('sheets', 'v4', http=credentials.authorize(Http()))

        result = service.spreadsheets().values().get(spreadsheetId=self.spreadsheet_id,
                                                     range=self.range_name).execute()
        return self._fill_missing_blank_cells(result.get('values', []))

    @staticmethod
    def _populate_merged_cells(row):
        populated_row = []
        previous_item = None
        for item in row:
            if previous_item and item == '':
                populated_row.append(previous_item)
            else:
                populated_row.append(item)
                previous_item = item
        return populated_row

    def _build_locations(self, locations, celestial_bodies):
        self.locations = {
            location: celestial_bodies[index]
            for index, location in enumerate(locations) if location
        }

    def _download_data_structure(self):
        values = self._get_spreadsheet_rows()
        locations, celestial_bodies = None, None
        if values:
            for row in values:
                if 'CRUSADER' in row:
                    celestial_bodies = self._populate_merged_cells(row)
                elif 'Port Olisar' in row:
                    if celestial_bodies:
                        self._build_locations(row, celestial_bodies)
                    locations = self._populate_merged_cells(row)
                elif row[0] and locations and celestial_bodies:
                    item_name = row[0]
                    item_prices = {}
                    for index, location in enumerate(locations):
                        if index == 0:
                            continue
                        if row[index]:
                            price = str(float(row[index].replace(",", ".")))
                            if index % 2:
                                transaction = 'Buy'
                            else:
                                transaction = 'Sell'
                            if item_prices.get(transaction):
                                if item_prices[transaction].get(price):
                                    item_prices[transaction][price].append(location)
                                else:
                                    item_prices[transaction][price] = [location]
                            else:
                                item_prices[transaction] = {price: [location]}
                    self.prices[item_name] = item_prices

    def _initiate_database(self):
        stored_data = self.database.get_trade_data()
        if stored_data:
            self.locations, self.prices = stored_data
        else:
            self._download_data_structure()
            self.database.save_trade_data(self.locations, self.prices)


class TradeAssistant(PricesStructure):

    @staticmethod
    def _get_filtered_locations(start_locations, lowest_buy_locations):
        new_buy_locations = []
        for location in lowest_buy_locations:
            if location in start_locations:
                new_buy_locations.append(location)
        return new_buy_locations

    def _get_allowed_locations(self, query):
        query = query.lower()
        return [location for location in self.locations.keys()
                if query in location.lower()
                or query in self.locations[location].lower()]

    @staticmethod
    def _exclude_locations(lowest_buy_locations, highest_sell_locations, exclude):
        assert isinstance(exclude, list)
        for no_go_location in exclude:
            if no_go_location in lowest_buy_locations:
                lowest_buy_locations.remove(no_go_location)
            if no_go_location in highest_sell_locations:
                highest_sell_locations.remove(no_go_location)

    def get_trade_routes(self, full_bay, budget, exclude=None, start_locations=None):
        if start_locations:
            start_locations = self._get_allowed_locations(start_locations)

        routes = []
        for item_name, prices in self.prices.items():
            lowest_buy = min([float(price) for price in prices["Buy"].keys()])
            lowest_buy_locations = prices["Buy"][str(lowest_buy)]   # type: list
            if start_locations:
                lowest_buy_locations = self._get_filtered_locations(start_locations, lowest_buy_locations)
            highest_sell = max([float(price) for price in prices["Sell"].keys()])
            highest_sell_locations = prices["Sell"][str(highest_sell)]

            if exclude:
                self._exclude_locations(lowest_buy_locations, highest_sell_locations, exclude)

            if len(lowest_buy_locations) < 1 or len(highest_sell_locations) < 1:
                continue

            bought_units = full_bay * 100
            spent_money = bought_units * lowest_buy
            if spent_money > budget:
                bought_units = budget / lowest_buy
                spent_money = budget

            money_after_trade = bought_units * highest_sell
            routes.append({
                'commodity': item_name,
                'invested money': spent_money,
                'income': round(money_after_trade - spent_money, 2),
                'bought units': round(bought_units, 2),
                'buy locations': ", ".join(["%s (%s)" % (location, self.locations[location])
                                            for location in lowest_buy_locations]),
                'buy price': lowest_buy,
                'sell locations': ", ".join(["%s (%s)" % (location, self.locations[location])
                                            for location in highest_sell_locations]),
                'sell price': highest_sell,
            })
        routes.sort(key=lambda item: item.get('income'), reverse=True)
        return routes


if __name__ == '__main__':
    assistant = TradeAssistant()
    with open("temp.json", 'w') as temp_file:
        temp_file.write(json.dumps(assistant.get_trade_routes(96, 2000, exclude=["Jumptown"]), indent=4))



from collections import namedtuple


Messages = namedtuple('Messages', [
    'ship_not_exists',
    'multiple_ships_found',
    'member_not_found',
    'member_ships_modified',
    'member_ships_invalid',
    'member_ship_not_found',
    'ship_price_unknown',
    'ship_from_report_not_found',
    'ship_price_report',
    'new_version',
    'road_map_not_found',
    'something_went_wrong'
])

Commands = namedtuple('Commands', [
    'help',
    'fleet',
    'member_ships',
    'add_ship',
    'remove_ship',
    'clear_member_ships',
    'prices',
    'ship',
    'compare',
    'releases',
    'roadmap',
    'trade',

])

messages_pl = Messages(
    ship_not_exists="%s, taki okręt nie istnieje!",
    multiple_ships_found="Niestety, nie jestem pewien czego dokładnie szukasz druhu. Czy chodziło Ci może o ...\n%s",
    member_not_found="Przykro mi, nie znalazłem takiej osoby.",
    member_ships_modified="Drogi %s, zanotowałem. Oto Twoja aktualna flota:",
    member_ships_invalid="%s, przykro mi lecz takie statki nie istnieją:",
    member_ship_not_found="%s, przykro mi lecz nie posiadasz tego statku.",
    ship_price_unknown="Wybacz Panie Bracie, lecz nie znam ceny okrętu *%s*  **%s**.",
    ship_from_report_not_found="Czołem!\nNie mogłem odnaleźć żadnych wieści o statku *%s*.",
    ship_price_report="Czołem!\n *%s* obecnie kosztuje *%s*",
    new_version="Czołem! Przynoszę Wam wieści:\n%s",
    road_map_not_found="Przykro mi, nic takiego nie znalazłem. Posiadam jednakże wiedzę o poniższych:\n```%s```",
    something_went_wrong="Coś poszło nie tak..."
)

commands_pl = Commands(
    help="pomoc",
    fleet="flota",
    member_ships="flota_członka",
    add_ship="dodaj",
    remove_ship="usuń",
    clear_member_ships="usuń moje statki",
    prices="ceny",
    ship="statek",
    compare="porównaj",
    releases="wersje",
    roadmap="obiecanki",
    trade="handel",
)

messages_en = Messages(
    ship_not_exists="%s, there's no such ship!",
    multiple_ships_found="Not sure what you mean. Maybe try one of this:\n%s",
    member_not_found="I'm sorry, there's no such member.",
    member_ships_modified="%s Here's your updated fleet:",
    member_ships_invalid="%s, sorry but those ships are not valid:",
    member_ship_not_found="%s, sorry but you don't have such ship.",
    ship_price_unknown="I'm sorry but I don't knowh price of *%s*  **%s**.",
    ship_from_report_not_found="Sorry, didn't find anything on ship *%s*.",
    ship_price_report="The price of *%s* is updated to *%s*",
    new_version="Howdy! I found some news:\n%s",
    road_map_not_found="Sorry, didn't find anything like that. You may try something like:\n```%s```",
    something_went_wrong="Something went wrong..."
)

commands_en = Commands(
    help="help",
    fleet="fleet",
    member_ships="member_ships",
    add_ship="add_ship",
    remove_ship="remove_ship",
    clear_member_ships="clear my ships",
    prices="prices",
    ship="ship",
    compare="compare",
    releases="releases",
    roadmap="roadmap",
    trade="trade",
)



import json

from sqlalchemy import create_engine
import sqlalchemy.exc
from sqlalchemy.orm import sessionmaker, exc

import settings
from . import database_models
from .my_logger import MyLogger


class DatabaseManager:
    def __init__(self, database_name=settings.DATABASE_NAME, log_file='database_mgr.log'):
        self.logger = MyLogger(log_file_name=log_file, logger_name="Database mgr logger", prefix="[DB]")
        database_engine = create_engine(settings.DATABASE_DIALECT % database_name, echo=False)
        database_models.Base.metadata.create_all(database_engine)

        session = sessionmaker(bind=database_engine)
        self.sql_alchemy_session = session()

    def add_and_get_member(self, user):
        query = self.sql_alchemy_session.query(database_models.Member)
        query = query.filter(database_models.Member.discord_id == user.id)
        try:
            self.logger.debug("Member %s already exists. Deleting..." % user.username)
            member = query.one()
            self.sql_alchemy_session.delete(member)
            self.sql_alchemy_session.commit()
            self.logger.debug("Deleted.")
        except exc.NoResultFound:
            self.logger.debug("Member %s does not exist." % user.username)
        self.logger.debug("Creating member %s" % user.username)
        member = database_models.Member(discord_id=user.id, name=user.username)
        self.sql_alchemy_session.add(member)
        self.sql_alchemy_session.commit()
        self.logger.debug("New member id is '%s'." % str(member.id))
        return member

    def get_all_members(self):
        return self.sql_alchemy_session.query(database_models.Member).all()

    @staticmethod
    def create_ship(ship_data, owner):
        return database_models.Ship(
            manufacturer=ship_data['manufacturer'],
            name=ship_data['name'],
            lti=ship_data.get('lti', False),
            package_id=ship_data.get('package_id'),
            owner_id=owner.id,
            owner=owner,
        )

    def update_member_ships(self, ships_data, owner):
        owner = self.add_and_get_member(owner)
        new_ships = []
        for ship_data in ships_data:
            new_ships.append(self.create_ship(ship_data, owner))
        self.sql_alchemy_session.add_all(new_ships)
        self.sql_alchemy_session.commit()

    def add_one_ship(self, ship_data, owner):
        member = self.get_member_by_discord_id(owner.id)
        if member is None:
            member = self.add_and_get_member(owner)
        ships = self.get_ships_dicts_by_member_id(member.id)
        if ships:
            ships.append(ship_data)
        else:
            ships = [ship_data]
        self.update_member_ships(ships, owner)

    def remove_one_ship(self, ship_data, owner):
        member = self.get_member_by_discord_id(owner.id)
        self.logger.debug("Removing ship %s" % str(ship_data))
        ships = self.get_ships_dicts_by_member_id(member.id)
        ship_removed = False
        if ships:
            self.logger.debug("Found %d member ships." % len(ships))
            for ship in ships:
                if ship['name'] == ship_data['name'] and ship['lti'] == ship_data['lti']:
                    ships.remove(ship)
                    ship_removed = True
                    break
            self.update_member_ships(ships, owner)
        return ship_removed

    def get_all_ships(self):
        return self.sql_alchemy_session.query(database_models.Ship).all()

    def get_member_by_name(self, member_name):
        member_name = member_name.lower()
        members_query = self.sql_alchemy_session.query(database_models.Member)
        members_query = members_query.filter(database_models.Member.name.like("%" + member_name + "%"))
        try:
            return members_query.one()
        except exc.NoResultFound:
            self.logger.debug("Member %s does not exist." % member_name)
        except exc.MultipleResultsFound:
            self.logger.debug("Too many members match name %s." % member_name)

    def get_member_by_discord_id(self, member_id):
        members_query = self.sql_alchemy_session.query(database_models.Member)
        members_query = members_query.filter(database_models.Member.discord_id == member_id)
        try:
            return members_query.one()
        except exc.NoResultFound:
            self.logger.debug("Member %s does not exist." % member_id)
        except exc.MultipleResultsFound:
            self.logger.debug("Too many members match name %s." % member_id)

    def get_ships_by_member_id(self, member_id):
        query = self.sql_alchemy_session.query(database_models.Ship)
        query = query.filter(database_models.Ship.owner_id == member_id).all()
        return query

    def get_ships_by_member_name(self, member_name):
        member = self.get_member_by_name(member_name)
        if member:
            return self.get_ships_by_member_id(member.id)

    def get_all_ships_dicts(self):
        return [ship.get_full_dict() for ship in self.get_all_ships()]

    def get_ships_dicts_by_member_id(self, member_id):
        ships = self.get_ships_by_member_id(member_id)
        if ships:
            return [ship.get_basic_dict() for ship in ships]

    def get_ships_dicts_by_member_name(self, member_name):
        ships = self.get_ships_by_member_name(member_name)
        if ships:
            return [ship.get_basic_dict() for ship in ships]

    def get_ships_summary(self):
        ships_names = set([ship.name for ship in self.get_all_ships()])
        result = []
        for name in ships_names:
            ship_query = self.sql_alchemy_session.query(database_models.Ship).filter(database_models.Ship.name == name)
            count = ship_query.count()
            ship_instances = ship_query.all()
            owners = set([ship.owner.name for ship in ship_instances])
            some_ship = ship_instances[0]
            result.append({
                'manufacturer': some_ship.manufacturer,
                'name': some_ship.name,
                'count': count,
                'owners': ", ".join(owners),
            })
        return result

    def update_versions(self, input_data):
        self.logger.debug("Updating PU and PTU version.")
        version_has_changed = False
        for key, new_value in input_data.items():
            query = self.sql_alchemy_session.query(database_models.Version)
            query = query.filter_by(name=key)
            try:
                old_data = query.one()
                if old_data.value != new_value:
                    self.logger.debug("There is new version. Updating.")
                    old_data.value = new_value
                    self.sql_alchemy_session.commit()
                    version_has_changed = True
            except exc.NoResultFound:
                self.sql_alchemy_session.add(database_models.Version(name=key, value=new_value))
                self.sql_alchemy_session.commit()
        return version_has_changed

    def save_rsi_data(self, ships_data):
        self.logger.debug("Updating RSI data.")
        ships_json = json.dumps(ships_data)
        query = self.sql_alchemy_session.query(database_models.RsiData)
        try:
            old_data = query.one()
            old_data.ships = ships_json
        except exc.NoResultFound:
            self.logger.debug("No RSI data in database. Creating object.")
            self.sql_alchemy_session.add(database_models.RsiData(ships=ships_json))
        except exc.MultipleResultsFound:
            self.logger.error("Multiple RSI objects in database!")
        self.sql_alchemy_session.commit()

    def get_rsi_data(self):
        query = self.sql_alchemy_session.query(database_models.RsiData)
        try:
            old_data = query.one()
            return json.loads(old_data.ships)
        except exc.NoResultFound:
            self.logger.warning("No RSI data in database!")
        except exc.MultipleResultsFound:
            self.logger.error("Multiple RSI objects in database!")

    @staticmethod
    def get_json_strings(*data):
        return [json.dumps(item) for item in data]

    @staticmethod
    def get_objects_from_json_strings(*data):
        return [json.loads(item) for item in data]

    def save_road_map(self, *road_map_data):
        self.logger.debug("Updating RSI data.")
        releases, categories, current_versions = self.get_json_strings(*road_map_data)

        query = self.sql_alchemy_session.query(database_models.RoadMap)
        try:
            old_data = query.one()
            old_data.releases = releases
            old_data.categories = categories
            old_data.current_versions = current_versions
        except exc.NoResultFound:
            self.logger.debug("No Road Map in database. Creating object.")
            self.sql_alchemy_session.add(database_models.RoadMap(
                releases=releases,
                categories=categories,
                current_versions=current_versions
            ))
        except exc.MultipleResultsFound:
            self.logger.error("Multiple Road Map objects in database!")
        self.sql_alchemy_session.commit()

    def get_road_map(self):
        query = self.sql_alchemy_session.query(database_models.RoadMap)
        try:
            old_data = query.one()
            return self.get_objects_from_json_strings(
                old_data.releases,
                old_data.categories,
                old_data.current_versions
            )
        except exc.NoResultFound:
            self.logger.warning("No Road Map object in database!")
        except exc.MultipleResultsFound:
            self.logger.error("Multiple Road Map objects in database!")

    def save_trade_data(self, *trade_data):
        self.logger.debug("Updating Trade Data.")
        locations, prices = self.get_json_strings(*trade_data)

        query = self.sql_alchemy_session.query(database_models.TradeData)
        try:
            old_data = query.one()
            old_data.locations = locations
            old_data.prices = prices
        except exc.NoResultFound:
            self.logger.debug("No Trade Data in database. Creating object.")
            self.sql_alchemy_session.add(database_models.TradeData(locations=locations, prices=prices))
        except exc.MultipleResultsFound:
            self.logger.error("Multiple Trade Data objects in database!")
        self.sql_alchemy_session.commit()

    def get_trade_data(self):
        query = self.sql_alchemy_session.query(database_models.TradeData)
        try:
            old_data = query.one()
            return self.get_objects_from_json_strings(
                old_data.locations,
                old_data.prices
            )
        except exc.NoResultFound:
            self.logger.warning("No Trade Data object in database!")
        except exc.MultipleResultsFound:
            self.logger.error("Multiple Trade Data objects in database!")

    def thread_is_new(self, thread_id, subject, url):
        try:
            self.sql_alchemy_session.add(database_models.FoundForumThreads(id=thread_id, subject=subject, url=url))
            self.sql_alchemy_session.commit()
            return True
        except sqlalchemy.exc.IntegrityError:
            self.sql_alchemy_session.rollback()
            return False

    def rsi_video_is_new(self, url):
        try:
            self.sql_alchemy_session.add(database_models.RsiLatestVideo(url=url))
            self.sql_alchemy_session.commit()
            return True
        except sqlalchemy.exc.IntegrityError:
            self.sql_alchemy_session.rollback()
            return False



import threading
import time

from disco.bot import Plugin
from tabulate import tabulate
import pafy

from .attachments_downloader import DiscordAttachmentHandler
from .database_manager import DatabaseManager
from .my_logger import MyLogger
from .rsi_parser import RsiDataParser
from .trade_assistant import TradeAssistant
import settings


class BaseBot(Plugin):
    main_channel_id = settings.CHANNELS['main']
    messages = settings.messages
    ship_data_headers = settings.SHIP_DATA_HEADERS

    def __init__(self, bot, config):
        super().__init__(bot, config)
        self.logger = MyLogger(log_file_name=settings.LOG_FILE, logger_name=settings.LOGGER_NAME, prefix="[BOT]")

        self.channel_main = self.client.api.channels_get(self.main_channel_id)
        self.bot_user = self.get_bot_user()

        self.database_manager = DatabaseManager(log_file=settings.LOG_FILE)

        self.attachments_handler = DiscordAttachmentHandler()

        self.rsi_data = RsiDataParser(7600, log_file=settings.LOG_FILE, database_manager=self.database_manager)
        self.report_ship_price_list = settings.REPORT_SHIP_PRICE_LIST

        self.monitoring_thread = threading.Thread(target=self.monitoring_procedure)
        self.monitoring_thread.start()

        self.trade = TradeAssistant(log_file=settings.LOG_FILE)

        self.help_message = self._get_help_message()

    def _get_help_message(self):
        header = ["Command", "Description"]
        rows = []
        for method in self.meta_funcs:
            command = " | ".join(" ".join(decorator['args']) for decorator in method.meta
                                 if decorator['type'] == 'command')
            if command:
                for decorator in method.meta:
                    description = decorator['kwargs'].get('docstring', "")
                    if description:
                        rows.append([command, description])
                        break
        rows.sort()
        return tabulate(rows, headers=header, tablefmt="presto")

    def get_bot_user(self):
        return self.bot.client.api.users_me_get()

    @staticmethod
    def mention_user(user):
        return '<@' + str(user.id) + '>'

    @staticmethod
    def mention_channel(channel):
        return '<#' + str(channel.id) + '>'

    def get_ship_data_from_name(self, ship_name):
        ship_data = self.rsi_data.get_ship(ship_name)
        if ship_data is None:
            found_ships = self.rsi_data.get_ships_by_query(ship_name)
            if len(found_ships) == 1:
                ship_data = found_ships[0]
            else:
                ship_data = found_ships
        return ship_data

    def show_invalid_ships(self, event, invalid_ships):
        event.channel.send_message(self.messages.member_ships_invalid % (self.mention_user(event.author)))
        event.channel.send_message("```%s```" % tabulate(invalid_ships, headers='keys', tablefmt="rst"))

    def update_fleet(self, event):
        for file in event.attachments.values():
            self.logger.debug("Checking file %s." % file.filename)
            try:
                if file.filename == "shiplist.json":
                    self.logger.debug("Getting ships list.")
                    ships = self.attachments_handler.get_ship_list(file.url, self.logger)
                    ships, invalid_ships = self.rsi_data.verify_ships(ships)
                    self.database_manager.update_member_ships(ships, event.author)

                    if invalid_ships:
                        self.show_invalid_ships(event, invalid_ships)
                    self.show_updated_member_ships(event)

            except Exception as unexpected_exception:
                self.logger.error(str(unexpected_exception))

    def clear_member_fleet(self, event):
        self.database_manager.update_member_ships([], event.author)

    def send_table_or_split_if_too_big(self, event, ships):
        table = tabulate(ships, headers='keys', tablefmt="presto")
        half_length = int(len(ships) / 2)
        if len(table) > 2000:
            self.send_table_or_split_if_too_big(event, ships[:half_length])
            self.send_table_or_split_if_too_big(event, ships[half_length:])
        else:
            event.msg.reply("```%s```" % table)
            self.logger.debug("Sending table with ships: %d, chars: %d." % (len(ships), len(table)))

    def get_ship_for_member(self, ship):
        ship_name = ship.replace("lti", "").strip()
        ship_data = self.get_ship_data_from_name(ship_name)
        if ship_data and isinstance(ship_data, dict):
            ship_data['lti'] = ship[-3:].lower() == "lti"
            return ship_data

    def show_updated_member_ships(self, event):
        ships = self.database_manager.get_ships_dicts_by_member_name(event.author.username)
        event.channel.send_message(self.messages.member_ships_modified % (self.mention_user(event.author)))
        event.channel.send_message("```%s```" % tabulate(ships, headers='keys', tablefmt="rst"))

    @staticmethod
    def get_ship_price_message(ship):
        return "*%s*  **%s**, price:  %s (+ VAT)" % (ship["manufacturer_code"], ship["name"], ship["price"])

    def format_ship_data(self, ship):
        table = [[key, ship.get(key, "unknown")] for key in self.ship_data_headers]
        return "%s\n```%s```\n" % (self.rsi_data.base_url + ship['url'], tabulate(table))

    def compare_ships_data(self, ships):
        table = [[key] for key in self.ship_data_headers]
        for ship in ships:
            if ship:
                for row in table:
                    row.append(ship.get(row[0], "unknown"))
        return "\n```%s```\n" % tabulate(table)

    def split_compare_if_too_long(self, ships):
        message = self.compare_ships_data(ships)
        if len(message) < 2000:
            messages = [message]
        else:
            half_length = int(len(ships) * 0.5)
            messages = self.split_compare_if_too_long(ships[:half_length])
            messages += self.split_compare_if_too_long(ships[half_length:])
        return messages

    def report_ship_price(self):
        for ship_name, price_limit in self.report_ship_price_list:
            ship_data = self.rsi_data.get_ship(ship_name)
            if ship_data is None:
                self.channel_main.send_message(self.messages.ship_from_report_not_found % ship_name)
            else:
                current_price = float(ship_data["price"][1:])
                if current_price > price_limit:
                    self.channel_main.send_message(self.messages.ship_price_report % (ship_name, ship_data["price"]))
                    self.report_ship_price_list.remove((ship_name, price_limit))

    def monitor_current_releases(self):
        current_releases = self.rsi_data.get_updated_versions()
        new_version_released = self.database_manager.update_versions(current_releases)

        if new_version_released:
            new_release_message = self.get_releases_message(current_releases)
            new_release_message = self.messages.new_version % new_release_message
            self.channel_main.send_message(new_release_message)

    def monitor_forum_threads(self):
        new_threads = self.rsi_data.get_forum_release_messages()
        if new_threads:
            self.channel_main.send_message(self.messages.new_version % "")
            for thread in new_threads:
                self.channel_main.send_message(thread)

    @staticmethod
    def get_releases_message(current_releases):
        return "PU Live: %s\nPTU: %s\n" % (current_releases.get('live'), current_releases.get('ptu'))

    def monitor_youtube_channel(self):
        latest_video_url = pafy.get_channel("RobertsSpaceInd").uploads[0].watchv_url
        if self.database_manager.rsi_video_is_new(latest_video_url):
            self.channel_main.send_message(latest_video_url)

    def monitoring_procedure(self):
        while True:
            self.monitor_current_releases()
            self.monitor_forum_threads()
            self.monitor_youtube_channel()
            self.report_ship_price()
            time.sleep(300)

    def show_no_road_map_data(self, event):
            self.logger.debug("No Roadmap data found...")
            event.msg.reply(self.messages.road_map_not_found %
                            tabulate(self.rsi_data.road_map.get_releases_and_categories(), tablefmt="fancy_grid"))

    @staticmethod
    def data_not_found(data, find):
        if find:
            return find.lower() not in data.lower()

    def show_road_map_data(self, event, data, find=None):
        if data:
            self.logger.debug("Showing Roadmap...")
            if isinstance(data, str):
                if self.data_not_found(data, find):
                    self.show_no_road_map_data(event)
                    return
                else:
                    event.msg.reply("```%s```" % tabulate(data, tablefmt="fancy_grid"))
            elif isinstance(data, dict):
                message_not_sent = True
                for key, value in data.items():
                    if self.data_not_found(key+str(value), find):
                        continue
                    event.msg.reply("```\n%s\n```" % tabulate(value, tablefmt="presto", headers=(key, "Task")))
                    message_not_sent = False
                if message_not_sent:
                    self.show_no_road_map_data(event)
        else:
            self.show_no_road_map_data(event)



from operator import itemgetter

from disco.bot import Plugin
from tabulate import tabulate

from .base_bot import BaseBot
from settings import additional_commands


class StarCitizenAssistant(BaseBot):

    @Plugin.listen('MessageCreate')
    def on_message_create(self, event):
        if event.attachments:
            self.logger.debug("The msg has an attachment. Checking if contains ship list..")
            self.update_fleet(event)

    @Plugin.command('help', docstring="Shows this help message.")
    @Plugin.command(additional_commands.help)
    def show_help(self, event):
        event.channel.send_message("```%s```" % self.help_message)

    @Plugin.command('fleet', parser=True, docstring="Organization fleet information. Try 'fleet -h' for more details.")
    @Plugin.command(additional_commands.fleet, parser=True)
    @Plugin.parser.add_argument("-o", "--order-by", action='store',
                                help="Choses columns for sorting, eg. '-o name,manufacturer'")
    @Plugin.parser.add_argument('-d', '--descending', action='store_true', help="Changes sorting to descending")
    @Plugin.parser.add_argument("-f", "--filter", action='store',
                                help="Choses columns to filter data, eg. '-f name=Herald,manufacturer=Drake'")
    @Plugin.parser.add_argument('-a', '--all-ships', action='store_true',
                                help="Do not stack same models. Show every single ship in seperate row.")
    @Plugin.parser.add_argument('-h', '--help', action='store_true', help="Show this help message.")
    def show_fleet(self, event, args):
        if args.all_ships:
            ships = self.database_manager.get_all_ships_dicts()
        else:
            ships = self.database_manager.get_ships_summary()

        if args.order_by:
            columns = args.order_by.split(",")
            columns.reverse()
        else:
            columns = ["name", "manufacturer"]
        for column in columns:
            ships = sorted(ships, key=itemgetter(column))
        if args.descending:
            ships.reverse()

        if args.filter:
            filters = args.filter.split(",")
            for item in filters:
                key, expected_value = item.split("=")
                ships = [ship for ship in ships if expected_value.lower() in str(ship[key]).lower()]

        if args.help:
            return event.msg.reply("```%s```" % event.parser.format_help())

        self.send_table_or_split_if_too_big(event, ships)

    @Plugin.command('member_fleet', '<member_name:str>', docstring="Lists ships owned by specified member.")
    @Plugin.command(additional_commands.member_ships, '<member_name:str>')
    def show_member_fleet(self, event, member_name):
        ships = self.database_manager.get_ships_dicts_by_member_name(member_name[:-1])
        if ships:
            event.channel.send_message("```%s```" % tabulate(ships, headers='keys', tablefmt="rst"))
        else:
            event.channel.send_message(self.messages.member_not_found)

    @Plugin.command('add_ship', '<ship:str...>', docstring="Manually add ship to fleet, e.g. 'add_ship Herald LTI'")
    @Plugin.command(additional_commands.add_ship, '<ship:str...>')
    def add_ship(self, event, ship):
        ship_data = self.get_ship_for_member(ship)
        if ship_data:
            self.database_manager.add_one_ship(ship_data, event.author)
            self.show_updated_member_ships(event)

    @Plugin.command('remove_ship', '<ship:str...>',
                    docstring="Manually remove ship from member fleet, e.g. 'remove_ship Herald LTI'")
    @Plugin.command(additional_commands.remove_ship, '<ship:str...>')
    def remove_ship(self, event, ship):
        ship_data = self.get_ship_for_member(ship)
        if ship_data:
            if self.database_manager.remove_one_ship(ship_data, event.author):
                self.show_updated_member_ships(event)
            else:
                event.channel.send_message(self.messages.member_ship_not_found % self.mention_user(event.author))

    @Plugin.command('clear my ships', docstring="Manually clear member fleet.")
    @Plugin.command(additional_commands.clear_member_ships)
    def clear_member_ships(self, event):
        self.clear_member_fleet(event)
        ships = self.database_manager.get_ships_by_member_name(event.author.username)
        if ships:
            event.channel.send_message(self.messages.something_went_wrong)

    @Plugin.command('prices', '<query:str...>', docstring="Ships prices in store credits, e.g. 'prices Cutlass'")
    @Plugin.command(additional_commands.prices, '<query:str...>')
    def check_ship_price(self, event, query):
        found_ships = self.rsi_data.get_ships_by_query(query)
        if len(found_ships) == 0:
            event.msg.reply(self.messages.ship_not_exists % (self.mention_user(event.author)))
        elif len(found_ships) > 24:
            event.msg.reply(self.messages.ship_not_exists % (self.mention_user(event.author)))
        else:
            prices_messages = []
            for ship in found_ships:
                try:
                    prices_messages.append(self.get_ship_price_message(ship))
                except KeyError:
                    prices_messages.append(self.messages.ship_price_unknown % (ship["manufacturer_code"], ship["name"]))
            event.msg.reply("\n".join(prices_messages))

    @Plugin.command('ship', '<query:str...>', docstring="Ship details, e.g. 'ship Cutlass Black'")
    @Plugin.command(additional_commands.ship, '<query:str...>')
    def check_ship_info(self, event, query):
        found_ship = self.get_ship_data_from_name(query)
        if isinstance(found_ship, list) and (1 < len(found_ship) < 7):
            for message in self.split_compare_if_too_long(found_ship):
                event.msg.reply(message)
        elif isinstance(found_ship, dict):
            event.msg.reply(self.format_ship_data(found_ship))
        else:
            event.msg.reply(self.messages.ship_not_exists % (self.mention_user(event.author)))

    @Plugin.command('compare', '<query:str...>',
                    docstring="Compare ships details, e.g. 'compare Cutlass Black,Freelancer MAX'")
    @Plugin.command(additional_commands.compare, '<query:str...>')
    def compare_ships(self, event, query):
        names = query.split(",")
        found_ships = []
        for name in names:
            ship_data = self.get_ship_data_from_name(name.strip())
            if isinstance(ship_data, list):
                found_ships += ship_data
            elif isinstance(ship_data, dict):
                found_ships.append(ship_data)
        if isinstance(found_ships, list) and len(found_ships) < 7:
            for message in self.split_compare_if_too_long(found_ships):
                event.msg.reply(message)
        else:
            event.msg.reply(self.messages.ship_not_exists % (self.mention_user(event.author)))

    @Plugin.command('releases', docstring="Current PU and PTU versions.")
    @Plugin.command(additional_commands.releases)
    def check_current_releases(self, event):
        current_releases = self.rsi_data.get_updated_versions()
        event.msg.reply(self.get_releases_message(current_releases))
        self.database_manager.update_versions(current_releases)

    @Plugin.command('roadmap', parser=True, docstring="Roadmap information. Try 'roadmap -h' for more details.")
    @Plugin.command(additional_commands.roadmap, parser=True)
    @Plugin.parser.add_argument("-v", "--version", action='store',
                                help="Details about specific release., eg. '-v 3.3.0'")
    @Plugin.parser.add_argument('-c', '--category', action='store',
                                help="Details about specified category., eg. '-c ships'")
    @Plugin.parser.add_argument('-f', '--find', action='store',
                                help="Find pages with specified expression., eg. '-f carrack'")
    @Plugin.parser.add_argument("-l", "--list", action='store_true',
                                help="Lists available categories and versions.")
    @Plugin.parser.add_argument('-h', '--help', action='store_true', help="Show this help message.")
    def road_map(self, event, args):
        self.logger.debug("Requested Roadmap.")
        if args.category and args.version:
            result = self.rsi_data.road_map.get_release_category_details(args.version, args.category)
            self.show_road_map_data(event, result)
        elif args.version:
            result = self.rsi_data.road_map.get_release_details(args.version)
            self.show_road_map_data(event, result, find=args.find)
        elif args.category:
            result = self.rsi_data.road_map.get_category_details(args.category)
            self.show_road_map_data(event, result, find=args.find)
        elif args.list:
            result = self.rsi_data.road_map.get_releases_and_categories()
            event.msg.reply("```%s```" % tabulate(result, headers='keys', tablefmt="fancy_grid"))
        elif args.help:
            event.msg.reply("```%s```" % event.parser.format_help())
        elif args.find:
            result = self.rsi_data.road_map.get()
            self.show_road_map_data(event, result, find=args.find)
        else:
            result = self.rsi_data.road_map.get_releases()
            event.msg.reply("```%s```" % tabulate(result, headers='keys', tablefmt="fancy_grid"))

    @Plugin.command('trade', parser=True, docstring="Trade assistant. Try 'trade -h' for more details.")
    @Plugin.command(additional_commands.trade, parser=True)
    @Plugin.parser.add_argument("-b", "--budget", action='store',
                                help="How much UEC you can spend, e.g. '-b 3000'")
    @Plugin.parser.add_argument('-c', '--cargo', action='store',
                                help="How much SCU you have available, e.g. '-c 46'")
    @Plugin.parser.add_argument('-e', '--exclude', action='store',
                                help="Exclude specific trade post, e.g. '-e Jumptown'")
    @Plugin.parser.add_argument('-s', '--start-location', action='store',
                                help="Find routes only starting from given location, e.g. '-s levski', '-s crusader'")
    @Plugin.parser.add_argument('-l', '--legal', action='store_true', help="Include only legal cargo, e.g. '-l'")
    @Plugin.parser.add_argument('-h', '--help', action='store_true', help="Show this help message.")
    def trade_route(self, event, args):
        if args.help:
            event.msg.reply("```%s```" % event.parser.format_help())
            return
        budget = 1000000000
        cargo = 1000000000
        exclude = set()
        if args.budget:
            budget = float(args.budget)
            if budget < 1:
                budget = 1
        if args.cargo:
            cargo = int(args.cargo)
            if cargo < 1:
                cargo = 1
        if args.exclude:
            exclude.add(args.exclude)
        if args.legal:
            exclude.add("Jumptown")

        result = self.trade.get_trade_routes(cargo, budget,
                                             exclude=list(exclude),
                                             start_locations=args.start_location)[:3]
        for route in result:
            event.msg.reply("```%s```" % tabulate(list(route.items()), tablefmt="presto"))



from setuptools import setup


with open("requirements.txt") as f:
    requirements = f.read().splitlines()
with open("README.md") as f:
    long_description = f.read()


setup(
    name='dastro_bot',
    version='1.0.7rc1',
    description='Discord bot for Star Citizen players',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://github.com/Mirdalan/discord_astro_bot',
    author='Michal Chrzanowski',
    author_email='michrzan@gmail.com',
    license='MIT',
    classifiers=[
        'Development Status :: 4 - Beta',

        'Intended Audience :: Developers',
        'Topic :: Games/Entertainment',

        'License :: OSI Approved :: MIT License',

        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    keywords='star citizen discord bot',
    install_requires=requirements,
    extras_require={
        ':sys_platform == "win32"': [
            'websocket-client==0.46.0'
        ],
        ':"linux" in sys_platform': [
            'websocket-client==0.44.0'
        ]
    },
    packages=['dastro_bot', 'dastro_bot._default_settings'],
    package_data={'': ['discord_bot.json', 'discord_bot.service']},
    include_package_data=True,
    python_requires='~=3.5',
)



import asyncio
import async_timeout
import aiohttp


class DiscordAttachmentHandler:
    def __init__(self):
        self.loop = asyncio.get_event_loop()

    @staticmethod
    async def fetch_json(session, url):
        async with async_timeout.timeout(10):
            async with session.get(url) as response:
                return await response.json()

    async def get_content(self, url):
        async with aiohttp.ClientSession() as session:
            return await self.fetch_json(session, url)

    def get_ship_list(self, file_url, logger):
        try:
            return self.loop.run_until_complete(self.get_content(file_url))
        except asyncio.TimeoutError:
            logger.error("Could not download attachment. Asyncio timeout error.")



import os
from shutil import copyfile

import sys


DEFAULT_FILES_DIR = "_default_settings"


def deploy_default_files(destination_dir="discord_bot"):
    module_dir = os.path.dirname(os.path.realpath(__file__))
    default_files_dir = os.path.join(module_dir, DEFAULT_FILES_DIR)
    for item in os.listdir(default_files_dir):
        source_file = os.path.join(default_files_dir, item)
        if os.path.isfile(source_file):
            copyfile(source_file, os.path.join(destination_dir, item))

def start_project(name="discord_bot"):
    destination_dir = os.path.join(os.getcwd(), name)
    os.mkdir(destination_dir)
    deploy_default_files(destination_dir)


if __name__ == '__main__':
    if len(sys.argv) == 2:
        start_project(sys.argv[1])
    else:
        print("Usage: python -m dastro_bot.install <directory_name>")
